#!/usr/bin/perl


#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014


use strict;
use DBI;
#connect to db
my $KEGG_sql = shift @ARGV;
my $HMDB_sql = shift @ARGV;
my $dbhk = DBI->connect("dbi:SQLite:dbname=$KEGG_sql",'','',{sqlite_use_immediate_transaction=>1});
my $dbhh = DBI->connect("dbi:SQLite:dbname=$HMDB_sql",'','',{sqlite_use_immediate_transaction=>1});
my $file = shift @ARGV;
open FILE, $file;
my @ids;
while ( my $line = <FILE> )
  {
  chomp $line;
  my $id = $line;
  push @ids, $id;
  }
my @kegg_ids;
my @hmdb_ids;
#get all ids from databases
my $sthk = $dbhk->prepare("SELECT ID from molecules ORDER BY ID");
$sthk->execute;
while ( my $row = $sthk->fetchrow_array )
  {
  push @kegg_ids, $row;
  }

my $sthh = $dbhh->prepare("SELECT ID from molecules ORDER BY ID");
$sthh->execute;
while ( my $row = $sthh->fetchrow_array )
  {
  push @hmdb_ids, $row;
  }


#here ids from the list of ids to remove are skipped, those that are not specified make it to @kegg or @hmdb_ids2

my @kegg_ids2;
OUTER: foreach my $id (@kegg_ids)
  {
  my $push = 0;
  foreach my $id2 (@ids)
    {
    if ($id eq $id2)
      {
      $push = 1;
      }
    }
  push @kegg_ids2, $id if ($push == 0);
  }

my @hmdb_ids2;
OUTER: foreach my $id (@hmdb_ids)
  {
  my $push = 0;
  foreach my $id2 (@ids)
    {
    if ($id eq $id2)
      {
      $push = 1;
      }
    }
  push @hmdb_ids2, $id if ($push == 0);
  }
print $#hmdb_ids, "\n";
print $#hmdb_ids2, "\n";
print $#kegg_ids, "\n";
print $#kegg_ids2, "\n";

#not necessaary but ensuring that ids go in increasing order incase ORDER by ID does something stupid
#or something.

@hmdb_ids2 = sort {$a cmp $b} @hmdb_ids2;
@kegg_ids2 = sort {$a cmp $b} @kegg_ids2;
#print join("\n",@hmdb_ids2);

my $h_total = $#hmdb_ids2;
my $k_total = $#kegg_ids2;
my $total = $h_total + $k_total;
print $total, "\n";
my $interval_h = shift @ARGV;
my $interval_k = shift @ARGV;
my $current = 0;
my $current_k = 0;
my $current_h = 0;
my $sthk = $dbhk->prepare("SELECT FORMULA from molecules WHERE ID = (?)");
my $sthh = $dbhh->prepare("SELECT FORMULA from molecules WHERE ID = (?)");
my %final;
print "\n";
while ( $current < $total )
  {
  #increase number from each db looked at but set to total if it would be more than there are total
  $current_k = $current_k + $interval_k;
  $current_h = $current_h + $interval_h;
  $current = $current + $interval_k + $interval_h;
  if ($current_k > $k_total)
    {
    $current_k = $k_total;
    }
  if ($current_h > $h_total) 
    {
    $current_h = $h_total;
    }
  #take the ids for the compounds being looked at, and get their formulas
  my @workingkids = @kegg_ids2[0..$current_k];
  my @workinghids = @hmdb_ids2[0..$current_h];
  my @formulas; 
  foreach my $id (@workingkids)
    {
    $sthk->execute($id);
    my $form = $sthk->fetchrow_array;
    push @formulas, $form;
    }
  foreach my $id (@workinghids)  
    {
    $sthh->execute($id);
    my $form = $sthh->fetchrow_array;
    push @formulas, $form;
    }
  my %cache;
  #look at all formulas, count number of unique and total formulas and compounds. unique formulas are non-isomeric compounds!
  foreach my $formula (@formulas)
    {
    $cache{$formula} ++;
    }
  my $unique_f = 0;
  my $total_f = 0;
  foreach my $key (keys %cache)
    {
    my $value = $cache{$key};
    if ($value == 1)
      {
      $unique_f++;
      $total_f++;
      }
    else 
      {
      $total_f = $total_f + $cache{$key}; 
      }
    }
  my $percent = ($unique_f / $total_f);
  $percent = 100*(1 - $percent);
  print $current_k, " : ", $current_h, " : ", $percent, " : ", "\n";
  %final = %cache;
  } 
