#!/usr/bin/perl

----------------
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
----------------

use strict;
use warnings;
use DBI;

my $translate = shift @ARGV;
my $stereos_1 = shift @ARGV;
my $stereos_2 = shift @ARGV;
my $stereos_3 = shift @ARGV;
my $count = 0;
my %stereos;
my $name_1;
my $name_2;
my $stereo;
my %translate;
my %test;

#open file with hdmb,kegg and alternative names. 
#Create mapping between kegg, hmdb names and the alternative names

open TRANS, $translate;
while ( my $line = <TRANS> )
  {
  chomp $line;
  my @names = split( /\$\$\$\$/, $line );
#  print join("\t", @names), "\n";
  $translate{$names[0]} = $names[2];
  $translate{$names[1]} = $names[2];
  }
close TRANS;


#read in KEGG stereo file remove any semicolons on names since 
#hmdb does not have semicolons
open FILE, $stereos_1;
while (my $line = <FILE>) 
  {
  chomp $line;
  if ( $count == 0 )
    {
    $name_1 = $line;
    $name_1 = "Unknown" if $name_1 eq ''; #not needed i don't think but there to cover all bases
    $name_1 =~ s/;//; #remove semicolon from names if there, should only be at end, if not it doesn't matter did it consistently
    $count ++;
    }
  elsif ( $count == 1 )
    {
    $name_2 = $line;
    $name_2 = "Unknown" if $name_2 eq '';
    $name_2 =~ s/;//;
    $count ++;
    }
  elsif ( $count == 2 )
    {
    #here we convert kegg names to alternative names if there is a valid mapping.
    $stereo = $line;
    if ( $stereo == 1 )
      {
      if ( exists $translate{$name_1} )
        {
        $name_1 = $translate{$name_1};
        }
      if ( exists $translate{$name_2} )
        {
        $name_2 = $translate{$name_2};
        }
      #record if they are stereos, should probably do this before translation for maximum efficiency
      $test{$name_1} = 1;
      $test{$name_2} = 1;
      $stereos{$name_1}{$name_2} = 1;
      $stereos{$name_2}{$name_1} = 1;
      }
    $count = 0;
    }
  else 
    {
    die "SOMETHING IS WRONG\n";
    }
  }
close FILE;

#read in HMDB stereos
open FILE2, $stereos_2;
while (my $line = <FILE2>)
  {
  chomp $line;
  if ( $count == 0 )
    {
    $name_1 = $line;
    $name_1 = "Unknown" if $name_1 eq ''; #cover all bases, shouldn't be needed though
    $count ++;
    }
  elsif ( $count == 1 ) 
    {
    $name_2 = $line;
    $name_2 = "Unknown" if $name_2 eq '';
    $count ++;
    }
  elsif ( $count == 2 )
    {
    $stereo = $line;
    if ( $stereo == 1 )
      {
      #convert names
      if ( exists $translate{$name_1} )
        {
        $name_1 = $translate{$name_1};
        }
      if ( exists $translate{$name_2} )
        {
        $name_2 = $translate{$name_2};
        }
      $test{$name_1} = 1;
      $test{$name_2} = 1; 
      $stereos{$name_1}{$name_2} = 1;
      $stereos{$name_2}{$name_1} = 1;
      }
    $count = 0;
    }
  else 
    {
    die "SOMETHING IS WRONG AGAIN\n";
    }
  }
close FILE2;

#read in stereo between db list
open FILE3, $stereos_3;
while ( my $line = <FILE3> )
  {
  chomp $line;
  if ( $count == 0 )
    {
    $name_1 = $line;
    $name_1 = "Unknown" if $name_1 eq '';
    $name_1 =~ s/;//;
    $count ++;
    }
  elsif ( $count == 1 )
    {
    $name_2 = $line;
    $name_2 = "Unknown" if $name_2 eq '';
    $name_2 =~ s/;//;
    $count ++;
    }
  elsif ( $count == 2 )
    {
    $stereo = $line;
    if ( $stereo == 1 )
      {
      if ( exists $translate{$name_1} )
        {
        $name_1 = $translate{$name_1};
        }
      if ( exists $translate{$name_2} )
        {
        $name_2 = $translate{$name_2};
        }
      $test{$name_1} = 1;
      $test{$name_2} = 1;
      $stereos{$name_1}{$name_2} = 1;
      $stereos{$name_2}{$name_1} = 1;
      }
    $count = 0; 
    }
  else
    {
    die "SOMETHING IS WRONG AGAIN AGAIN\n";
    }
  }
close FILE3;

#now connect to DBs and retrieve all names in order of ID from each

my $kegg_total;
my $hmdb_total;
my $SQLite_kegg = shift @ARGV;
my $SQLite_hmdb = shift @ARGV; 
my @keggnames;
my @hmdbnames;
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite_kegg",'','',{sqlite_use_immediate_transaction=>1});
my $sth = $dbh->prepare("SELECT NAME from molecules ORDER BY ID");
$sth->execute;
while ( my $row = $sth->fetchrow_array )
  {
  $row =~ s/;//;
  $row = $translate{$row} if ( exists $translate{$row} );
  push @keggnames, $row;
  $kegg_total ++;
  }

my $dbh2 = DBI->connect("dbi:SQLite:dbname=$SQLite_hmdb",'','',{sqlite_use_immediate_transaction=>1});
my $sth2 = $dbh2->prepare("SELECT NAME from molecules ORDER BY ID");
$sth2->execute;
while ( my $row = $sth2->fetchrow_array )
  {
  $row = $translate{$row} if ( exists $translate{$row} );
  push @hmdbnames, $row;
  $hmdb_total ++;
  }
#figure out total number of names
my $total = $kegg_total+$hmdb_total;
my $interval_kegg = shift @ARGV;
my $interval_hmdb = shift @ARGV;
$|++;
my $value_k = 0;
my $value_h = 0;
printf("%-20s %-20s %-20s %-20s %-20s\n", "KEGG_entries#", "HMDB_entries#", "UniqueEntries#", "Stereos#", "UniqueSteros#");
while ( ($value_k + $value_h) < $total )
  {
  my @unique_names;
  my @unique_stereos;
  $value_k = $value_k + $interval_kegg;
  $value_h = $value_h + $interval_hmdb;
  if ( $value_k > $kegg_total )
    {
    $value_k = $kegg_total;
    }
  if ( $value_h > $hmdb_total )
    {
    $value_h = $hmdb_total;
    }
  my @names_2;
  my %stereo_list;
  my @stereos;
  my $is_stereo = 0;
  #for the number of compounds being looked at, put the amount for each db onto an array. 
  foreach my $index (0..$value_k)
    {
    push @names_2, $keggnames[$index] if ($keggnames[$index] ne '');
    }
  foreach my $index (0..$value_h)
    {
    push @names_2, $hmdbnames[$index] if ($hmdbnames[$index] ne  '');
    }
  my %seen_names;
  @unique_names = grep { ! $seen_names{$_} ++; } (@names_2);
#  print join("\n", @names_2), "\n";
#  die;

  #now go through names, check if in one list there is a name that is mapped to another name in either list 
  #to be a stereoisomer, if so record that it is a stereo isomer.
  foreach my $name_1 (@names_2)
    {
    $is_stereo = 0;
    next if (! exists $test{$name_1});
    next if (exists $stereo_list{$name_1});
    foreach my $name_2 (@names_2)
      {
      last if ($is_stereo == 1);
#      print $name_2, "\n";
      next if (! exists $test{$name_2});
      if (exists $stereos{$name_1}{$name_2})
        {
        $is_stereo = 1;
        }
      elsif (exists $stereos{$name_2}{$name_1})
        {
        $is_stereo = 1;
        }
      }
    push @stereos, $name_1 if ($is_stereo == 1);
    #record all stereos but keep track of how many unique ones there are this is different than number of stereos total. 
    my %seen_stereos;
    @unique_stereos = grep { ! $seen_stereos{$_} ++; } @stereos;
    }
  printf("%-20s %-20s %-20s %-20s %-20s\n", $value_k, $value_h, $#unique_names+1, $#stereos+1, $#unique_stereos+1);
#  print $value_k, " : ", $value_h, " : ", $#unique_names+1, " : ", $#stereos+1, " : ", $#unique_stereos+1, "\n";
  }









  
