#!usr/bin/perl

#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014

use lib '../MODULES';
use DBI;
use strict;
use MolFile;
use Molecule;

my $SQLite = shift @ARGV;
my $molfile = shift @ARGV;

my $compounds = ( MolFile->new( "File" => $molfile, "Type" => 'compound' )->parseCompound());
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite",'','',{AutoCommit=>0, RaiseError=>1});

my $count;
foreach my $compound (values %$compounds)
  {
  $count++;
  }
print $count . "\n";

my $make_compound_table_string = "CREATE TABLE IF NOT EXISTS molecules (NAME text, ATOMS text, BONDS text, FORMULA text, EXTENDEDFORMULA text, ID text PRIMARY KEY, MOLFILE text)";
my $make_table = $dbh->prepare($make_compound_table_string);
$make_table->execute;

my $insert_molecule_string = "INSERT INTO molecules (NAME,ATOMS,BONDS,FORMULA,EXTENDEDFORMULA,ID,MOLFILE) values (?,?,?,?,?,?,?)";
my $insert_molecule = $dbh->prepare($insert_molecule_string);
$count = 0;
foreach my $compound (values %$compounds) 
  {
  my $atom_entry; #the atom and bond entries aren't used for anything and can be removed to save space
  my $bond_entry; #they may however be used for something in the future so I'm keeping them in. 
  $count ++; 
  $compound->addHydrogens();
  foreach my $atom (@{$$compound{Atoms}})
    {
    $atom_entry .= join(",",map{$$atom{$_};} (sort keys %$atom));
    $atom_entry .= "|";
    }
  foreach my $bond (@{$$compound{Bonds}})
    {
    $bond_entry .= join(",",map{$$bond{$_};} (sort keys %$bond));
    $bond_entry .= "|";
    }
  print $count . "\n";
  $insert_molecule->execute($$compound{Name}, $atom_entry, $bond_entry, $$compound{Formula}, $$compound{ExtendedFormula}, $$compound{ID}, $$compound{String});
  }
$dbh->commit();
$dbh->disconnect;
