#!/usr/bin/perl
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
use lib "../MODULES";
use DBI;
use strict;
use Array::Utils qw(:all);
use MolFile;
use Molecule;
use UllmannImproved2;


#
#READ IN ARGV
#

my $enumerator = Ullmann->new();
my $SQLite = shift @ARGV;
my $fgroup = shift @ARGV;
my $ID = shift @ARGV;
my $result_file = shift @ARGV;

#
#PARSE FUNCTIONALS, CONNECT TO DATABASE
#

my $fgroups = (MolFile->new("File"=>$fgroup, "Type" => 'functional')->parse());
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite",'','',{sqlite_use_immediate_transaction=>1});

#
#MAKE MATRIX FOR FGROUPS
#

my @fgroups = values %$fgroups;
foreach my $fgroup (@fgroups)
  {
  $fgroup->makeAdjacencyMatrix();
  }

#
#FIND SUPER FGROUPS AND NORMAL FGROUPS
#

my @super_fgroups;
my @normal_fgroups;

foreach my $functional (@fgroups)
  {
  if ($functional->Name =~ /SUPER/)
    {
    push @super_fgroups, $functional;
    }
  else
    {
    push @normal_fgroups, $functional;
    }
  }

#
#LIST FUNCTIONAL NAMES
#

my @normal_functional_names;
foreach my $fgroup (@normal_fgroups)
  {
  my $name = $fgroup->Name();
  push @normal_functional_names, "Inclusive$name", "Overlapping$name", $name;
  }

my @super_functional_names;
foreach my $fgroup (@super_fgroups)
  {
  my $name = $fgroup->Name();
  push @super_functional_names, "Super$name";
  }

#print join("\n",@normal_functional_names) . "\n";
#print join("\n",@super_functional_names), "\n";

#
#GET MOLFILE FOR COMPOUND
#

my $mol_file_array = [split("\n", &retrieve_molfile($ID))];
my $molecule_name = &retrieve_name($ID);
my $compounds = MolFile::parseCompound2($mol_file_array);
my $compound = $$compounds{$molecule_name};

#
#FIND NORMAL FGROUPS
#
#find how many planes of symmetry each fgroup has. If it has N, for X instances of the group, X*N will be detected so we need to divide by X to correct. 
#This is no longer necessary as it is now built in, but it may be used in the future. 
my %normal_multiplicity;
foreach my $functional (@normal_fgroups)
  {
  my $M_matrix = $functional->makeMappingMatrix3($functional,[]);
  my $isomorphs = $enumerator->enumerate($M_matrix, $functional, $functional, 'long', []);
  $normal_multiplicity{$functional->Name} = $#$isomorphs + 1;
  }



$compound->addHydrogens();
$compound->makeAdjacencyMatrix();
my %count_hash;
my @overlap;
map { $count_hash{$_} = 0; } (@normal_functional_names);
my $functional_groups = [];
# print "\n";
foreach my $functional (@normal_fgroups)
  {
#  print $functional->Name, "\n";

  my $array = [];
  my $M_matrix = $compound->makeMappingMatrix3($functional,$array);
  my $isomorphs = $enumerator->enumerate( $M_matrix, $functional, $compound, 'long', $array);
#  print @$isomorphs, "\n";
  foreach my $iso (@$isomorphs)
    {
    my $new_iso = [];
    my $starting_index = 0;
#    print join(" ",@$iso), "\n";
    foreach my $index (0..$#$iso)
      {
      $$new_iso[$starting_index] = $$iso[$index] if ! grep( {$_ == $index } (@{$$functional{Context}}));
      $starting_index = $starting_index + 1 if ! grep( {$_==$index} (@{$$functional{Context}}));
      }
    $iso = $new_iso;
    }
  foreach my $iso (@$isomorphs)
    {
#    print join(" ",@$iso), "\n";
    push @$iso, $functional->Name;
    push @{$$compound{Functionals}}, [@$iso];
    }
  }

foreach my $a (@{$$compound{Functionals}})
  {
  my @array;
  my @subgraph;
  my $diff = 0;
  my $subgraph = 0;
  my $overlapping = 0;
  my $same = 0;
  my $name_1 = pop @$a;
#  print "A: ", join(" ",@$a), "\n";
  foreach my $b (@{$$compound{Functionals}})
    {
#    print "\tB: ", join(" ",@$b), "\n";
    my $name_2; 
    if ($a == $b)
      {
      $name_2 = $name_1;
      }
    else
      {
      $name_2 = pop @$b; 
      }
    
    
#    my $name_2 = pop @$b if ($a != $b);
    my @diffa = array_minus(@$b,@$a);
    my @diffb = array_minus(@$a,@$b);
    my $diff2 = 0;
    my $subgraph2 = 0;
    my $overlapping2 = 0;
    my $same2 = 0;

    if ( array_minus(@$b, @diffa) == 0 and array_minus(@$a,@diffb) == 0 ) #these don't overlap at all, they share no indices
      {
#      print "\t\tDIFF\n";
      $diff2 ++;
      }
    elsif ( @diffa != 0 and @diffb == 0 ) #one functional group indices is a subset of the other, it is a subgraph
      {
#      print "\t\tSUBG\n";
      $subgraph2 ++;
      }
    elsif ( @diffa !=0 and @diffb != 0 and @diffa = @$b and @diffa = @$a ) #there is overlap but neither are subgraphs
      {
#      print "\t\tOVER\n";
      $overlapping2 ++;
      }
    elsif ( @diffa == 0 and @diffb == 0 ) #either these are the same groups or there is a group it is a subgraph of that it is 'overlapping' with as it shares vertices
      {
      if ($name_1 eq $name_2)
        {
        $same2 ++; 
#        print "\t\tSAME\n";
        }
      else 
        {
        $overlapping2 ++;
        }
      }
    push @$b, $name_2 if ($a != $b);
    $diff = $diff + $diff2;
    $subgraph = $subgraph + $subgraph2;
    $overlapping = $overlapping + $overlapping2;
    $same = $same + $same2;
    push @overlap, $name_1,".",$name_2,":" if ($overlapping2 != 0);
    }
#  print "SAME:",$same, "\n", "OVER:",$overlapping,"\n";;
  if ( $subgraph == 0 and $overlapping == 0 )# and $same == 1 )
    {
#    $count_hash{$name_1} += (1/$normal_multiplicity{$name_1});;
    $count_hash{$name_1} += (1/$same); #if it isn't overlapping or a subgraph, it must be a distinct instances. Divide by same as a funtional group with n planes of symmetry will find itself N times for each single instances. So divide by the number of times you found the same vertices and the same group name to correct. 
    }
  elsif ( $subgraph != 0 )
    {
    $count_hash{"Inclusive$name_1"} += (1/$same); # (1/$normal_multiplicity{$name_1});
    }
  elsif ( $overlapping != 0)
    {
    $count_hash{"Overlapping$name_1"} += (1/$same); # (1/$normal_multiplicity{$name_1});;
    }
  elsif ( $same > 1 )
    {
#    print $same, "\n";
#    print "they be the same\n";
    $count_hash{$name_1} += (1/$same) #(1/$normal_multiplicity{$name_1});
    }
  push @$a, $name_1;
  }
$$compound{"Groups"} = \%count_hash;
$compound->ExtendedFormula($compound->Formula . "&&");

foreach my $key (keys %count_hash)
  {
  $compound->ExtendedFormula($compound->ExtendedFormula . $key . $count_hash{$key} . "&&");#  if ($count_hash{$key} != 0);
  }

#print $compound->ExtendedFormula, "\n";
#now find super functional groups
foreach my $functional_2 (@super_fgroups)
  {
  my $M_matrix = $compound->makeMappingMatrix3($functional_2);
  my $isomorphs = $enumerator->enumerate($M_matrix, $functional_2, $compound, 'long', []);
  my $count = $#$isomorphs + 1;
  my $M_matrix2 = $functional_2->makeMappingMatrix3($functional_2);
  my $isomorphs2 = $enumerator->enumerate($M_matrix2, $functional_2, $functional_2, 'long', []);
  my $multiplicity = $#$isomorphs2 + 1;
  my $name = $functional_2->Name;
  $compound->ExtendedFormula($compound->ExtendedFormula . "Super$name" . ($count/$multiplicity) . "&&");
  }

if ($result_file eq '')
  {
  $result_file = $ID . ".out";
  }
open FILE, ">$result_file";
print FILE $compound->ExtendedFormula,"\n";

sub retrieve_molfile
  {
  my $molecule_ID = shift @_;
  my $sth = $dbh->prepare("SELECT MOLFILE from molecules WHERE ID = '$molecule_ID'");
  $sth->execute;
  my $molfile = $sth->fetchrow_array;
  $sth->finish;
  return $molfile;
  }

sub retrieve_name
  {
  my $ID = shift @_;
  my $sth = $dbh->prepare("SELECT NAME from molecules WHERE ID = '$ID'");
  $sth->execute;
  my $molecule_name = $sth->fetchrow_array;
  $sth->finish;
  return $molecule_name;
  }

