#!/usr/bin/perl

#Matrix.pm
# Contains the methods for Matrix, an object that represents a matrix


#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014


package Matrix;

#Load module to enable accessor autogeneration

use base 'Class::Accessor::Fast';

#Load module to allow subroutine caching


#Load custom modules

#use Vector;

#Load pragma

use strict;
#use warnings; 

#Create prototypical matrix object

my @matrix_fields =
  (
  "Matrix"        => "0",   #placeholder for an array ref
  "Matrix_string" => "0",   #string representation of the matrix
  );

#Make accessors for Matrix class

Matrix->mk_accessors( ( grep { $_ ne "0"; } @matrix_fields ) ); 

#Constructs new object of class 'Matrix'

sub new
  {
  my $proto = shift @_;
  my $self = { @matrix_fields, @_};
  $self = bless ( $self, ref($proto) || $proto);
  $self->Matrix_string( join( map ( { join('[',join(',',@$_),"]\n");} (@$self{Matrix}))) . "\n");
  #$self->stringMatrix();
  return $self;
  }

#multiply matrix takes two matrix object, $self and $other and returns their product as a matrix object

sub improvedMultiply
  {
  my $self = shift @_;
  my $other = shift @_;
  my $product_matrix = [ map { [map {0;} (0..$#{${$other->Matrix}[0]}) ] } (0..$#{$self->Matrix}) ];
  map { my $i = $_; map { my $j = $_; map { $$product_matrix[$i][$j] = $$product_matrix[$i][$j] + ( ${$self->Matrix}[$i][$_] * ${$other->Matrix}[$_][$j] ); } (0..$#{${$self->Matrix}[0]}) } (0..$#{${$other->Matrix}[0]}) } (0..$#{$self->Matrix}) ; 
  return Matrix->new("Matrix"=>$product_matrix);
  }

#take $self, a matrix object, and transpose it and set it equal to a new matrix object and return.

sub transpose
  {
  my ($self, $new_matrix) = (shift @_, []);
#  my $self = shift @_;
#  my $new_matrix = [];
  foreach my $i (0..$#{$self->Matrix})
    {
    foreach my $j (0..$#{${$self->Matrix}[0]})
      {
      $$new_matrix[$j][$i] = ${$self->Matrix}[$i][$j];
      }
    }
  return Matrix->new("Matrix" => $new_matrix);
  }

#taking $self a matrix object, count up all elements equal to 1. 
sub count 
  { 
  my $count = 0;
  my $self = shift @_;
  foreach my $i (0..$#{$self->Matrix})
    {
    foreach my $j (0..$#{${$self->Matrix}[0]})
      {
      $count ++ if (${$self->Matrix}[$i][$j] == 1);
      }
    }
  return $count;
  }
        
#taking $self a matrix object, make a string that when printed will look like the matrix and save it as Matrix_string.

sub stringMatrix
  {
  my $self = shift @_;
  $self->Matrix_string( join( '',  map ( { join('','[', join(',', @$_ ),']', "\n" )  ; } (@{$self->Matrix}) ) ) . "\n" );
  }


#both clones will duplicate a matrix object. clone2 is faster than clone but also harder to read. $self is a matrix object

sub clone
  {
  my $self = shift @_;
  return Matrix->new( "Matrix" => [ map ( { my $row = $_; [ map ( { ( $_ ) ; } (@$row) ) ] } (@{$self->{Matrix}}) ) ] );
  }

sub clone2
  {
  my $self = shift @_;
  return Matrix->new( "Matrix" => [ map ( { [ @$_ ] } (@{$self->{Matrix}}) ) ] ); 
  }



#no longer in used, included for completeness

sub rearrange
  {
  my $self = shift @_;
  my $mappings = shift @_;
  return Matrix->new("Matrix" => [ map { my $row = $_; [ map { ${$self->Matrix}[$row][$_]; } (@$mappings) ]; } (@$mappings) ] );
  }

#equate 1, equate 2 and equate 3 are the same functionality but 2 should be faster than 3 and 1 slower than 2 or 3. . 

#check if selected regions of self matrix, other matrix specified by the mapping vector are the same, mapping is 1d array, self and other are matrix objects (blessed 2d arrays)

sub equate
  {
  my $self = shift @_;
  my $other = shift @_; 
  OUTER: foreach my $row_index (0..($#{$self->Matrix}))
    {
    foreach my $column_index (0..($#{${$self->Matrix}[0]}))
      {
      return 0 if ( ${$self->Matrix}[$row_index][$column_index] != ${$other->Matrix}[$row_index][$column_index] );
      }
    }
  return 1;
  }

#check if selected regions of self matrix, other matrix specified by the mapping vector are the same, mapping is 1d array, self and other are matrix objects (blessed 2d arrays)
sub equate2
  {
  my ($self, $other, $mapping) = @_;
  my ($sm, $om) = ( $$self{Matrix}, $$other{Matrix} );
  for my $row ( 0..$#$mapping ) 
    {
    my ($smrow, $omrow) = ($$sm[$row], $$om[$$mapping[$row]]);
    for my $col (0..$#$mapping)
      {
      return 0 if $$smrow[$col] != $$omrow[$$mapping[$col]];
      }
    }
  return 1;
  }

#check if selected regions of self matrix, other matrix specified by the mapping vector are the same, mapping is 1d array, self and other are matrix objects (blessed 2d arrays)

sub equate3
  {
  my $self = shift @_;
  my $other = shift @_;
  my $mapping = shift @_;
  foreach my $row_index (0..$#$mapping)
    {
    next if ($$mapping[$row_index] == -1);
    return 0 if (grep { ($$mapping[$_] != -1) && (${$self->Matrix}[$row_index][$_] != ${$other->Matrix}[$$mapping[$row_index]][$$mapping[$_]]); } (0..$#$mapping));
    }
  return 1;
  }


sub slicer
  {
  my $self = shift @_;
  my $depth = shift @_;
  my $column = shift @_;
  my $new_matrix = [];
  foreach my $D ( 0..$depth )
    {
    foreach my $C ( 0..$column )
      {
      $$new_matrix[$D][$C] = ${$self->Matrix}[$D][$C];
      }
    }
  return Matrix->new("Matrix" => $new_matrix); 
  }
  
sub copy # only works for matrices with the same number of rows.
  {
  my ($self,$other) = @_;
  map { @{$self->{Matrix}[$_]} = @{$other->{Matrix}[$_]}; } (0..$#{$other->{Matrix}});
  return $self;
  }

1;




