#!/usr/bin/perl
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
use lib '../MODULES';
use strict;
use MolFile;
use DBI; 
use UllmannImproved2;

my $enumerator = Ullmann->new();
my ($SQLite_file, $list_file) = @ARGV;
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite_file",'','',{sqlite_use_immediate_transaction=>1});
open FILE, $list_file or die "CANNOT OPEN FILE!!!\n";
my $count = 0;
while (my $line = <FILE>)
  {
  $count ++;
#  print STDERR $count, "\n";
  chomp $line;
  my @ids = split(":", $line);
#  print $ids[0], "\t", $ids[1], "\n";
  my $mol_file_array = [split("\n", (&retrieve_molfile($ids[0]) . "\n" . &retrieve_molfile($ids[1])))]; #get and read molfiles from sqlite
  my $compounds = MolFile::parseCompound2($mol_file_array); #here's the reading part
  map { $_->addHydrogens(); $_->makeAdjacencyMatrix(); $_->color(6); } (values %$compounds); #color the nodes
  my ($molecule_name_1, $molecule_name_2) = (&retrieve_Name($ids[0]), &retrieve_Name($ids[1])); #get the names based on id, could get them from the read molfiles but this is just as easy

  my $compound1 = $$compounds{$molecule_name_1}; #get compound 1 and 2 separate for ease of use
  my $compound2 = $$compounds{$molecule_name_2};
  my $bond_count_1 = @{$$compound1{Bonds}};#get bond count
  my $bond_count_2 = @{$$compound2{Bonds}};
  if ($bond_count_1 != $bond_count_1)
    {
    print $molecule_name_1, "\n", $molecule_name_2. "\n", "0\n"; #diff # of bonds can't be stereoisomers
    next;
    }
  elsif ($compound1->totalcolor ne $compound2->totalcolor)
    {
    print $molecule_name_1, "\n", $molecule_name_2, "\n", "0\n"; #diff connectivity due to color difference, can't be stereoisomers
    next;
    }
  print $molecule_name_1, "\n", $molecule_name_2, "\n"; #could be stereo
    my $M_matrix = $$compounds{$molecule_name_1}->makeMappingMatrixColor($$compounds{$molecule_name_2}); #make adj matrix using color 
  my $array = [];
#  print $M_matrix->stringMatrix(), "\n";

  my $isomorphs = $enumerator->enumerateShort( $M_matrix, $$compounds{$molecule_name_1}, $$compounds{$molecule_name_2}, 'short', $array); #actually test connectivity
  if ($#$isomorphs == -1)
    {
    print "0\n"; #was not stereo
    }
  else 
    {
    print "1\n"; #is stereoe
    }  
  }
 
my %cache;
sub retrieve_molfile
  {
  my $molecule_ID = shift @_;
  my $sth = $dbh->prepare("SELECT MOLFILE from molecules WHERE ID = '$molecule_ID'");
  $sth->execute;
  my $molfile = $sth->fetchrow_array;
  $sth->finish;
  return $molfile;
  }

sub retrieve_Name
  {
  my $ID = shift @_; 
  if (exists $cache{$ID}{Name})
    {
    return $cache{$ID}{Name};
    }
  my $sth = $dbh->prepare("SELECT NAME from molecules WHERE ID = '$ID'");
  $sth->execute;
  my $molecule_name = $sth->fetchrow_array;
  $sth->finish;
  $cache{$ID}{Name} = $molecule_name;
  return $molecule_name;
  }

$dbh->disconnect();
