#!/usr/bin/perl
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
use strict;
#use warnings;

my %skip_cache;
my $print_conversion_table = 0;
my %formulas;
my $total_formulas;

sub make_entry
  {
  if ( $print_conversion_table == 0 )
    {
    printf ("%-20s%-60s\n\n", "INDEX:", "FUNCTIONAL GROUP:");
    }
  my $count = 0;
  my $hash = shift @_;
  my $formula = shift @_;
  $formulas{$formula} ++;
  my $array = [];
  push @$array, $formula;
  LOOP: foreach my $key (sort keys %$hash)
    {
    next LOOP if (exists $skip_cache{$key});
    if ( $key =~ /^Super/ or $key =~ /^Overlapping/ or $key =~ /^Inclusive/)
      {
      $skip_cache{$key};
      next;
      }
      
    $count ++ if ($print_conversion_table == 0);
#    printf ("%-40s%-20s%-20s\n",$key, $$hash{$key}, $$hash{"Inclusive$key"}); 
    push @$array, ($$hash{$key}+$$hash{"Inclusive$key"});
    if ($print_conversion_table == 0)
      {
      printf ("%-20s%-60s\n", $count, $key);
      }
    }
  return $array;
  }

my $entries = [];
open FILE, shift @ARGV;
while ( my $line = <FILE> )
  {
  chomp $line;
  my @array = split(/&&/, $line);
  my $formula = shift @array;
  my %hash;
  foreach my $element (@array)
    {
    my ($group, $value) = split( /\.mol/, $element);
    $hash{$group} = $value;
    }
  my $matrix_row = &make_entry(\%hash, $formula); 
#  print join(",",@$matrix_row), "\n";
  push @$entries, $matrix_row;
  $print_conversion_table = 1;
  }
close FILE;

my $total = $#$entries + 1;
my $columns = $#{${$entries}[0]};

my $strategies;
my $iterations = 0;
my $iteration_limit = shift @ARGV;;
my $keep_x = shift @ARGV;
my $cutoff = shift @ARGV;
my $first_cut = shift @ARGV;
my $mode = shift @ARGV;
my @keys = keys %formulas;
my $total_formulas = $#keys+1;
print $total_formulas, "\n";
$keep_x --;
push @$strategies, [0];
print "Strategy Depth: 0\n";
print &analyze_strategy([0]), "\n";
my $first = 1;
while ( $iterations < $iteration_limit and $#$strategies != -1 )
  {
  print $#$strategies + 1, "\n";
  my $new_strategies;
  $iterations ++; 
  print "Strategy Depth: $iterations\n";
  foreach my $strategy (@$strategies)
    {
    my $start = pop @$strategy if ( $first != 1 );
    foreach my $index (1..$columns)
      {
      my $new_strategy = &copy_array($strategy);
      push @$new_strategy, $index;
      my $unique = &analyze_strategy($new_strategy);
      push @$new_strategy, $unique;
      my $score = $unique - $start;
      if ( $score < $cutoff )
        {
        next;
        }
      push @$new_strategies, $new_strategy;
      }
    }
  print "SORTING\n";
  my %unique;
  foreach my $strategy (@$new_strategies)
    {
    last if ($first == 1);
    my $value = pop @$strategy;
    my @a = sort( {$a <=> $b} @$strategy);
    my $string = join(":",@a);
    $unique{$string} = $value;
    }
  my $array = [];
  foreach my $key (keys %unique)
    {
    last if ($first == 1);
    my $string = $key;
    my $strategy = [split(":",$string)];
    push @$strategy, $unique{$key};
    push @$array, $strategy;
    }
  $new_strategies = $array if ( $first != 1);;
  print $#$new_strategies, "\n";
  $new_strategies = [ sort { $b->[-1] <=> $a->[-1] ; } @$new_strategies ] if ($#$new_strategies != -1); 
  $new_strategies = [ @$new_strategies[0..$keep_x] ] if ($#$new_strategies != -1 and $first == 0 and $keep_x < $#$new_strategies);
  $new_strategies = [ @$new_strategies[0..$first_cut] ] if ( $first == 1);
  if ($#$new_strategies == -1)
    {
    $iterations --;
    die "$iterations is sufficient depth\n";
    }   
  foreach my $strategy (@$new_strategies)
    {
    print join(",",@$strategy), "\n";
#    pop @$strategy;
    }
  $strategies = $new_strategies;
  $first = 0;
  }

sub analyze_strategy
  {
  my $strategy = shift @_; 
  my @extended_formulas;
  foreach my $row (@$entries)
    {
    my $string;
    foreach my $index (@$strategy)
      {
      $string = $string."$$row[$index]#";
      }
    push @extended_formulas, $string;
    }
  my %hash;
  foreach my $formula (@extended_formulas)
    {
#    print STDERR $formula, "\n";
    $hash{$formula} ++;
    }
  my $unique = 0;
  $total = 0;
  foreach my $value (values %hash)
    {
    $unique ++ if ($value == 1);
    $total = $total + $value;
    }
  my $percent_u = ( $unique / $total ) * 100;
  if ( $mode eq "FORMULA" )
    {
    my $total_formulas = keys %hash;
#    print $unique, "\n";
#    print $total_formulas, "\n";
    $percent_u = ( $unique / $total_formulas ) * 100;
    }


  return $percent_u;
  }

sub copy_array
  {
  my $array = shift @_;
  my $result = [ map { $_ } @$array ];
  return $result;
  }



