#!/usr/bin/perl
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
use strict;
use warnings; 
use lib '../MODULES';
use MolFile;
use DBI;
use UllmannImproved2;

my $enumerator = Ullmann->new();
my ($SQLite_kegg, $SQLite_hmdb, $list) = @ARGV;
my $dbh_kegg = DBI->connect("dbi:SQLite:dbname=$SQLite_kegg",'','',{sqlite_use_immediate_transaction=>1});
my $dbh_hmdb = DBI->connect("dbi:SQLite:dbname=$SQLite_hmdb",'','',{sqlite_use_immediate_transaction=>1});
open FILE, $list or die "CANNOT OPEN FILE!!!\n";
my $count = 0;

while ( my $line = <FILE> )
  {
  $count ++;
  print STDERR $count, "\n";
  chomp $line;
  my @ids = split(":",$line);
  my $kegg_mol = [split( "\n", &retrieve_molfile($ids[0],$dbh_kegg))];
  my $hmdb_mol = [split( "\n", &retrieve_molfile($ids[1],$dbh_hmdb))];
  my $kegg_compounds = MolFile::parse_newKEGG2($kegg_mol);
  my $hmdb_compounds = MolFile::parseCompound2($hmdb_mol);
  my $kegg_name = &retrieve_name($ids[0], $dbh_kegg);
  my $hmdb_name = &retrieve_name($ids[1], $dbh_hmdb);
  print $kegg_name, "\n", $hmdb_name, "\n";
  next if ((lc $kegg_name) eq (lc $hmdb_name));
  my $kegg_compound = $$kegg_compounds{$kegg_name};
  my $hmdb_compound = $$hmdb_compounds{$hmdb_name};
  $kegg_compound->addHydrogens();
  $hmdb_compound->addHydrogens();
  $kegg_compound->makeAdjacencyMatrix();
  $hmdb_compound->makeAdjacencyMatrix();
  $kegg_compound->color(5);
  $hmdb_compound->color(5);
  print "0\n" and next if $kegg_compound->totalcolor ne $hmdb_compound->totalcolor;
  my $M_matrix = $kegg_compound->makeMappingMatrixColor($hmdb_compound);
  my $isomorphs = $enumerator->enumerateShort( $M_matrix, $hmdb_compound, $kegg_compound, 'short', [] );
  if ($#$isomorphs == -1)
    {
    print "0\n";
    }
  else
    {
    print "1\n";
    }
  }

sub retrieve_molfile
  {
  my $id = shift @_;
  my $dbh = shift @_;
  my $sth = $dbh->prepare("SELECT MOLFILE from molecules WHERE ID = '$id'");
  $sth->execute;
  my $molfile = $sth->fetchrow_array;
  $sth->finish;
  return $molfile;
  }

sub retrieve_name
  {
  my $id = shift @_;
  my $dbh = shift @_;
  my $sth = $dbh->prepare("SELECT NAME from molecules WHERE ID = '$id'");
  $sth->execute;
  my $molecule_name = $sth->fetchrow_array;
  return $molecule_name;
  }
