#!/usr/bin/perl


#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014


use lib "../MODULES";
use DBI;
use MolFile;
use Molecule;
use UllmannImproved2;

my $fgroup = shift @ARGV;
my $SQLite = shift @ARGV;
my $fgroups = (MolFile->new("File"=>$fgroup, "Type" => 'functional')->parse());
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite",'','',{sqlite_use_immediate_transaction=>1});

my @fgroups = values %$fgroups;
foreach my $fgroup (@fgroups)
  {
  $fgroup->makeAdjacencyMatrix();
  }

my @ids;
my $sth = $dbh->prepare("SELECT ID from molecules");
$sth->execute;

while ( my $row = $sth->fetchrow_array )
  {
  push @ids, $row;
  }
my %highest;
foreach my $functional (@fgroups)
  {
  $highest{$functional->Name} = 0; #start at maximum value of m = 0 for all groups
  }

foreach my $id (@ids)
  {
#  print $id, "\n";
  my $mol_file_array = [split("\n", &retrieve_molfile($id))];
  my $compounds = MolFile::parse_newKEGG2($mol_file_array);
  my $molecule_name = &retrieve_name($id);
  my $compound = $$compounds{$molecule_name};
  foreach my $functional (@fgroups)
    {
    my $M_matrix = $compound->makeMappingMatrix3($functional,[]);
    my $count = $M_matrix->count();
    if ( $highest{$functional->Name} < $count ) #if m is greater than current maximum set maximum to m
      {
      $highest{$functional->Name} = $count;
      }
    }
  }

foreach my $key (keys %highest)
  {
  print $key, "::", $highest{$key}, "\n";
  }

sub retrieve_name
  {
  my $ID = shift @_;
  my $sth = $dbh->prepare("SELECT NAME from molecules WHERE ID = '$ID'");
  $sth->execute;
  my $molecule_name = $sth->fetchrow_array;
  $sth->finish;
  return $molecule_name;
  }

sub retrieve_molfile
  {
  my $molecule_ID = shift @_;
  my $sth = $dbh->prepare("SELECT MOLFILE from molecules WHERE ID = '$molecule_ID'");
  $sth->execute;
  my $molfile = $sth->fetchrow_array;
  $sth->finish;
  return $molfile;
  }

$dbh->disconnect();
