#!/usr/bin/perl
#       Copyright Joshua M. Mitchell and Hunter N.B. Moseley, 2014. All rights reserved.
#       Written by Joshua Mitchell 5/1/2014
use lib '../MODULES';
use strict;
use MolFile;
use DBI;
use UllmannImproved2;

#use POSIX qw(strftime);
#use Time::HiRes qw/time/;


my ($array, $enumerator) = ([], Ullmann->new());
my ($SQLite_file, $ID_1, $ID_2, $result_dir) = @ARGV;
my $dbh = DBI->connect("dbi:SQLite:dbname=$SQLite_file",'','',{sqlite_use_immediate_transaction=>1});



my $mol_file_array = [split("\n", (&retrieve_molfile($ID_1) . "\n" . &retrieve_molfile($ID_2)))]; #get molfiles

my $compounds = MolFile::parse_newKEGG2($mol_file_array); #read molfiles
my ($molecule_name_1, $molecule_name_2) = (&retrieve_Name($ID_1), &retrieve_Name($ID_2)); #get names

map { $_->addHydrogens(); $_->makeAdjacencyMatrix() } ( values %$compounds );
my $M_matrix = $$compounds{$molecule_name_1}->makeMappingMatrix2($$compounds{$molecule_name_2}); 

my $isomorphs;
$isomorphs = $enumerator->enumerateShort( $M_matrix, $$compounds{$molecule_name_1}, $$compounds{$molecule_name_2}, 'short', $array); #test connectivity


my $file_name = "$ID_1%$ID_2";

my $results_file = ">$result_dir$file_name.result";
open( RESULTS,  $results_file ) or die "CANNOT OPEN $results_file";
print RESULTS "$molecule_name_1\n$molecule_name_2\n";
if ($#$isomorphs == -1)
  {
  print RESULTS "0\n";
#  print " : 0\n";
  }
else 
  {
  print RESULTS "1\n";
#  print " : 1\n";
  }

sub retrieve_molfile
  {
  my $molecule_name = shift @_;
  my $sth = $dbh->prepare("SELECT MOLFILE from molecules WHERE ID = '$molecule_name'");
  $sth->execute;
  my $molfile = $sth->fetchrow_array;
  $sth->finish;
  return $molfile;
  }

sub retrieve_Name
  {
  my $ID = shift @_;
  my $sth = $dbh->prepare("SELECT NAME from molecules WHERE ID = '$ID'");
  $sth->execute;
  my $molecule_name = $sth->fetchrow_array;
  $sth->finish;
  return $molecule_name;
  }

$dbh->disconnect();
close( RESULTS );
